/*
 * Example Accordion CSS 
 */

 div.toggler {
    width: 300pxpx;
 }
 
a.toggler {
  display: block;
  background-color: white;
  opacity: 0.7;
  -moz-opacity:0.7;
  text-align: left;
  width: 300px;
  border: 0px;
  font-weight:bolder;
  text-decoration: none;
}

a.toggler:hover {
    background-color: yellow;
}



