# Release information about AccordionWidget

version = "1.0"

description = "An accordion Widget based on minikit javascript code"
long_description = """
AccordionWidget transforms a kid template containing a lot of A/DIV elements pairs into a visual accordion widget. 
"""
author = "Luigi S. Palese"
email = "ultrabit@gmail.com"
copyright = "Luigi S. Palese 2007"

# if it's open source, you might want to specify these
# url = "http://yourcool.site/"
# download_url = "http://yourcool.site/download"
license = "MIT"
