/* minikit.widget.ContextMessage */
.minikit-contextmessage-header,
.minikit-contextmessage-main {
	cursor: default;
}
.minikit-contextmessage-header {
	background-color: white;
	padding-top: 8px;
	padding-left: 8px;
	padding-right: 8px;
	border-top: 1px solid gray;
	border-left: 1px solid gray;
	border-right: 1px solid gray;
}
.minikit-contextmessage-main {
	background-color: white;
	padding-left: 8px;
	padding-right: 8px;
	padding-bottom: 8px;
	border-left: 1px solid gray;
	border-right: 1px solid gray;
}
.minikit-contextmessage-footer {
	background-image: url(bottom.gif);
}
.minikit-contextmessage-button {
	height: 16px;
	width: 16px;
	background-image: url(close.gif);
	background-repeat: no-repeat;
}
.minikit-contextmessage-button-hover {
	height: 16px;
	width: 16px;
	background-image: url(close-hover.gif);
	background-repeat: no-repeat;
}
.minikit-contextmessage-icon {
	height: 16px;
	width: 16px;
	background-image: url(alert.gif);
	background-repeat: no-repeat;
}
